//	Fungus.h - A fungus.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FUNGUS_H_
#define FUNGUS_H_

#include <vector>
#include <string>
#include <stdint.h>

class Drawer;
class GamePlay;

//------------------------------------------------------------------------------
///	A collision bounding box.
struct BoundingBox
{
	///	Position within the fungus image.
	float x;
	///	Position within the fungus image.
	float y;
	///	Size within the fungus image.
	float w;
	///	Size within the fungus image.
	float h;
};

//------------------------------------------------------------------------------
///	A fungus.
class Fungus
{
  public:
	///	Constructor.
	/*!
		\param fungusName This fungi's name.
		\param fungusImage This fungi's image.
		\param w The width of the image.
		\param h The height of the image.
		\param bounds The collision bounding boxes for this fungus.
		\param fungusScale Whether this fungi is on the ceiling (-1) or the
		floor (1).
	 */
	Fungus(const std::string& fungusName,
		   uint32_t fungusImage,
		   float w,
		   float h,
		   const std::vector<BoundingBox>& bounds,
		   float fungusScale);
	///	Destructor.
	~Fungus();

	///	Moves the fungus.
	/*!
		\return Returns true if the fungus has moved offscreen.
	 */
	bool move(GamePlay *gp);
	///	Draws the fungus.
	void draw(Drawer *d);

	///	Decrements this fungi's health.
	void decrementHealth();
	///	Sets easy mode.
	void setEasyMode(bool val);

	///	Returns this fungi's name.
	const std::string& getName() const {return name;};
	///	Returns this fungi's position.
	float getX() const {return xPos;};
	///	Returns this fungi's position.
	float getY() const {if(scale > 0.0f) return yPos; else return (yPos-height);};
	///	Returns this fungi's width.
	float getWidth() const {return width;};
	///	Returns this fungi's height.
	float getHeight() const {return height;};
	///	Returns this fungi's health.
	int getHealth() const {return health;};
	///	Returns this fungi's bounding boxes.
	const std::vector<BoundingBox>& getBounds() const {return collisionBoxes;};

	///	Fungi max health.
	static int maxHealth;
	///	Whether fungi are attacking the player yet.
	static bool attacking;
  private:
	///	The fungi's name.
	std::string name;
	///	The fungi's image.
	uint32_t image;

	///	The fungi's position.
	float xPos;
	///	The fungi's position.
	float yPos;
	///	The fungi's width.
	float width;
	///	The fungi's height.
	float height;
	///	How to scale the image on the y-axis.
	float scale;
	///	The fungi's health.
	float health;

	///	Used to shoot the pink spores.
	int shootCount;

	///	The collision bounding boxes.
	std::vector<BoundingBox> collisionBoxes;

	///	Whether we're in easy mode or not.
	bool easyMode;
};

#endif
