//	GamePlay.cpp - The main gameplay state
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef DEBUG
#pragma warning(disable : 4786) //Stupid MS headers...
#endif

#include "GamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "Random.h"
#include "Spore.h"
#include "Weapon.h"
#include "Bullet.h"
#include "GameCounter.h"
#include "KillerSpore.h"
#include "Sounds.h"

#include <cmath>
#include <sstream>

using namespace std;

//------------------------------------------------------------------------------
uint32_t GamePlay::background[3] = {0, 0, 0};
uint32_t GamePlay::midground[3] = {0, 0, 0};
uint32_t GamePlay::foreground[3] = {0, 0, 0};
uint32_t GamePlay::scanningBar = 0;
uint32_t GamePlay::pinkSpore = 0;
uint32_t GamePlay::boulder = 0;
uint32_t GamePlay::particles[3] = {0, 0, 0};
uint32_t GamePlay::font = 0;
uint32_t GamePlay::bigFont = 0;

//------------------------------------------------------------------------------
GamePlay::GamePlay(Application *app, bool easy):
GameState(app),
fungiCount(0),
fungiHealthCount(8),
explosionIndex(0),
bossTime(false),
sporeCount(0),
pinkSporeCount(0),
boulderCount(0),
killerGenerated(false),
shake(0.0f),
winWipe(1.0f),
bossWipe(1.0f),
paused(false),
easyMode(easy)
{
	int i;
	int which;
	float x = -32.0f;

	if(easyMode)
	{
		player.setEasyMode(true);
		boss.setEasyMode(true);
	}

	Fungus::maxHealth = 1;
	Fungus::attacking = false;

	helpText.setPosition(80, 650, 1150);
	helpText.setColour(0.0f, 0.0f, 0.0f);
	if(font)
		helpText.setFont(font);

	//Foreground rocks.
	for(i=0;i<8;++i)
	{
		Rock rock;

		rock.x = x;
		rock.y = -96.0f;
		rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
		which = (int)(Random::getGlobal().getRandomFloat() > 0.5f);
		foreRocks[which].push_back(rock);

		rock.x = x;
		rock.y = (800.0f-72.0f);
		rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
		which = (int)(Random::getGlobal().getRandomFloat() > 0.5f);
		foreRocks[which].push_back(rock);

		x += 192.0f;
	}

	//Midground rocks.
	x = -48.0f;
	for(i=0;i<10;++i)
	{
		Rock rock;

		rock.x = x;
		rock.y = -8.0f;
		rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
		which = (int)(Random::getGlobal().getRandomFloat() > 0.5f);
		midRocks[which].push_back(rock);

		rock.x = x;
		rock.y = (800.0f-220.0f);
		rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
		which = (int)(Random::getGlobal().getRandomFloat() > 0.5f);
		midRocks[which].push_back(rock);

		x += 160.0f;
	}

	//Background rocks.
	x = -96.0f;
	for(i=0;i<15;++i)
	{
		Rock rock;

		rock.x = x;
		rock.y = 280.0f;
		rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
		farRocks.push_back(rock);

		rock.x = x;
		rock.y = 100.0f;
		rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
		which = (int)(Random::getGlobal().getRandomFloat() > 0.5f);
		backRocks[which].push_back(rock);

		rock.x = x;
		rock.y = (800.0f-352.0f);
		rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
		which = (int)(Random::getGlobal().getRandomFloat() > 0.5f);
		backRocks[which].push_back(rock);

		x += 104.0f;
	}

	//Setup particle generators.
	for(i=0;i<MaxExplosions;++i)
	{
		explosions[i].setLifespan(30.0f);
		explosions[i].setDensity(8.0f);
		explosions[i].setColour(0.75f, 0.75f, 0.75f, 0.75f);
		explosions[i].setDirection(-1.0f, -0.33f);
	}

	if(font)
	{
		for(i=0;i<MaxExplosions;++i)
		{
			explosions[i].addImage(particles[0]);
			explosions[i].addImage(particles[1]);
			explosions[i].addImage(particles[2]);
		}
	}
}

//------------------------------------------------------------------------------
GamePlay::~GamePlay()
{
	int i;

	for(i=0;i<fungi.size();++i)
		delete fungi[i];

	for(i=0;i<spores.size();++i)
		delete spores[i];

	for(i=0;i<killerSpores.size();++i)
		delete killerSpores[i];

	for(i=0;i<weapons.size();++i)
		delete weapons[i];

	for(i=0;i<bullets.size();++i)
		delete bullets[i];
}

//------------------------------------------------------------------------------
void GamePlay::draw(Drawer *d)
{
	int i, j;
	float x, y;
	float tempf;
	float shakeVal = 0.0f;

	if(!font)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::background1_png, Images::background1_pngSize);
		background[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::background2_png, Images::background2_pngSize);
		background[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::background3_png, Images::background3_pngSize);
		background[2] = d->loadImage(data.data, data.width, data.height);


		data = imLoad.loadImage(Images::midground1_png, Images::midground1_pngSize);
		midground[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::midground2_png, Images::midground2_pngSize);
		midground[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::midground3_png, Images::midground3_pngSize);
		midground[2] = d->loadImage(data.data, data.width, data.height);


		data = imLoad.loadImage(Images::foreground1_png, Images::foreground1_pngSize);
		foreground[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::foreground2_png, Images::foreground2_pngSize);
		foreground[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::foreground3_png, Images::foreground3_pngSize);
		foreground[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::scanningbar_png, Images::scanningbar_pngSize);
		scanningBar = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::spore3_png, Images::spore3_pngSize);
		pinkSpore = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::boulder_png, Images::boulder_pngSize);
		boulder = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::debris1_png, Images::debris1_pngSize);
		particles[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::debris2_png, Images::debris2_pngSize);
		particles[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::debris3_png, Images::debris3_pngSize);
		particles[2] = d->loadImage(data.data, data.width, data.height);

		fungusFactory.loadImages(d);


		font = d->loadFont("BEACHTYP.TTF", 40.0f);
		bigFont = d->loadFont("BEACHTYP.TTF", 64.0f);

		helpText.setFont(font);

		for(i=0;i<MaxExplosions;++i)
		{
			explosions[i].addImage(particles[0]);
			explosions[i].addImage(particles[1]);
			explosions[i].addImage(particles[2]);
		}
	}

	//Do non-drawing updating.
	if(!paused && !player.getDead())
	{
		if(boss.getX() > (1280.0f-512.0f))
		{
			///	Generate new fungi if necessary.
			--fungiCount;
			if((fungiCount <= 0) && (winWipe >= 1.0f))
			{
				fungi.push_back(fungusFactory.getRandomFungus());
				fungi[fungi.size()-1]->setEasyMode(easyMode);

				tempf = (fungi[fungi.size()-1]->getHeight()/400.0f);
				fungiCount = fungi[fungi.size()-1]->getWidth();
				fungiCount *= tempf;
				fungiCount += 60;
				fungiCount += (int)(Random::getGlobal().getRandomFloat() * 59.9999f);
				if(tempf < 0.33f)
					fungiCount -= (int)(Random::getGlobal().getRandomFloat() * 59.9999f);
			}
		}

		///	Generate new spores if necessary.
		--sporeCount;
		if((sporeCount <= 0) && (winWipe >= 1.0f) && !bossTime)
		{
			float numFungiKilled;
			float countOffset;

			if(easyMode)
				countOffset = (float)(fungusFactory.getNumFungusTypes()) * 8.0f;
			else
			{
				countOffset = (float)(fungusFactory.getNumFungusTypes() -
									  GameCounter::getInstance().getCount("FungiScanned"));
				countOffset *= 4.0f;
			}

			sporeCount = 30 + countOffset;
			sporeCount += (int)(Random::getGlobal().getRandomFloat() * (float)(sporeCount - 0.0001f));

			numFungiKilled = (float)(GameCounter::getInstance().getCount("FungiKilled"));
			if(numFungiKilled < 15.0f)
			{
				if((Random::getGlobal().getRandomFloat() * 15.0f) > numFungiKilled)
					spores.push_back(new Spore());
				else
				{
					killerSpores.push_back(new KillerSpore());
					if(!killerGenerated)
					{
						killerGenerated = true;
						helpText.trigger("Message from Command:\n Watch out for the red spores. We believe they may be harmful.");
					}
				}
			}
			else
			{
				killerSpores.push_back(new KillerSpore());
				if(!killerGenerated)
				{
					killerGenerated = true;
					helpText.trigger("Message from Command:\n Watch out for the red spores.  We believe they are harmful.");
				}
			}
		}

		//Generate new boulders if necessary.
		if(pinkSporeCount >= 8)
		{
			--boulderCount;
			if(boulderCount <= 0)
			{
				Boulder boulder;

				if(!bossTime)
					boulderCount = 50;
				else
					boulderCount = 90;
				if(easyMode)
					boulderCount += 60;
				boulderCount += (int)(Random::getGlobal().getRandomFloat() * 29.9999f);

				shake = 1.0f;

				boulder.x = Random::getGlobal().getRandomFloat() * 1200.0f;
				boulder.y = -128.0f;
				boulder.rot = Random::getGlobal().getRandomFloat() * 360.0f;

				boulders.push_back(boulder);
				Sounds::getInstance().playSound(Sounds::FungusDie);
			}
		}

		if(boss.getX() > (1280.0f-512.0f))
		{
			//Far background rocks.
			for(j=0;j<farRocks.size();++j)
			{
				Rock& rock = farRocks[j];

				rock.x -= 4.0f;
				if((rock.x + 256.0f) < 0.0f)
				{
					rock.x = 1280.0f;
					rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
				}
			}

			for(i=0;i<2;++i)
			{
				//Background rocks.
				for(j=0;j<backRocks[i].size();++j)
				{
					Rock& rock = backRocks[i][j];

					rock.x -= 3.0f;
					if((rock.x + 256.0f) < 0.0f)
					{
						rock.x = 1280.0f;
						rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
					}
				}

				//Midground rocks.
				for(j=0;j<midRocks[i].size();++j)
				{
					Rock& rock = midRocks[i][j];

					rock.x -= 2.0f;
					if((rock.x + 256.0f) < 0.0f)
					{
						rock.x = 1280.0f;
						rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
					}
				}

				//Foreground rocks.
				for(j=0;j<foreRocks[i].size();++j)
				{
					Rock& rock = foreRocks[i][j];

					rock.x -= 1.0f;
					if((rock.x + 256.0f) < 0.0f)
					{
						rock.x = 1280.0f;
						rock.image = (int)(Random::getGlobal().getRandomFloat() * 2.9999f);
					}
				}
			}
		}

		if(boss.getX() > (1280.0f-512.0f))
		{
			//Move the fungi.
			for(i=(fungi.size()-1);i>=0;--i)
			{
				if(fungi[i]->move(this))
				{
					delete fungi[i];
					fungi.erase(fungi.begin()+i);
				}
			}
		}

		//Move the spores.
		for(i=(spores.size()-1);i>=0;--i)
		{
			if(spores[i]->move())
			{
				delete spores[i];
				spores.erase(spores.begin()+i);
			}
		}

		//Move the pink spores.
		for(i=(pinkSpores.size()-1);i>=0;--i)
		{
			PinkSpore& spore = pinkSpores[i];

			spore.x += spore.xDir * 4.0f;
			spore.y += spore.yDir * 4.0f;
			if((spore.x < -32.0f) || (spore.x > 1280.0f) ||
			   (spore.y < 0.0f) || (spore.y > 800.0f))
			{
				pinkSpores.erase(pinkSpores.begin()+i);
			}
		}

		//Move the killer spores.
		for(i=(killerSpores.size()-1);i>=0;--i)
		{
			if(killerSpores[i]->move())
			{
				delete killerSpores[i];
				killerSpores.erase(killerSpores.begin()+i);
			}
		}

		//Move the boulders.
		for(i=(boulders.size()-1);i>=0;--i)
		{
			Boulder& b = boulders[i];

			if(boss.getX() > (1280.0f-512.0f))
				b.x -= 1.0f;
			b.y += 4.0f;
			b.rot += 2.0f;
			if(b.y > 800.0f)
				boulders.erase(boulders.begin()+i);
		}

		if(bossTime)
			boss.move(this);

		//Move the player's ship.
		player.move();

		//Move the weapons accordingly.
		for(i=0;i<weapons.size();++i)
			weapons[i]->move(this, player.getX(), player.getY());

		//Move the bullets.
		for(i=(bullets.size()-1);i>=0;--i)
		{
			if(bullets[i]->move())
			{
				delete bullets[i];
				bullets.erase(bullets.begin()+i);
			}
		}

		//Move explosions.
		for(i=0;i<MaxExplosions;++i)
			explosions[i].move();

		//Do any collision detection.
		playerFungiCollision();
		playerSporeCollision();
		if((winWipe >= 1.0f) && (bossWipe >= 1.0f))
		{
			playerKillerSporeCollision();
			playerPinkSporeCollision();
			playerBoulderCollision();
		}
		bulletFungiCollision();
		playerFungiScanning();
		if(bossTime)
			bulletBossCollision();
	}

	//If the player's just been killed, delete their weapons.
	if(player.getDead() && (weapons.size() > 0))
	{
		for(i=0;i<weapons.size();++i)
			delete weapons[i];
		weapons.clear();
	}

	if(shake > 0.0f)
	{
		shakeVal = sinf(shake * 16.0f * 6.283185f) * 4.0f * shake;
		d->translate(shakeVal, 0.0f);
		shake -= 0.01f;
	}

	//Draw far background rocks.
	for(j=0;j<farRocks.size();++j)
	{
		Rock& rock = farRocks[j];

		d->drawImage(background[rock.image], rock.x, rock.y, 1.0f, 1.0f, 0.6f, 0.5f, 0.75f);
	}

	//Draw background rocks.
	for(i=0;i<2;++i)
	{
		for(j=0;j<backRocks[i].size();++j)
		{
			Rock& rock = backRocks[i][j];

			d->drawImage(background[rock.image], rock.x, rock.y);
		}
	}

	//Draw midground rocks.
	for(i=0;i<2;++i)
	{
		for(j=0;j<midRocks[i].size();++j)
		{
			Rock& rock = midRocks[i][j];

			d->drawImage(midground[rock.image], rock.x, rock.y);
		}
	}

	//Draw explosions.
	for(i=0;i<MaxExplosions;++i)
		explosions[i].draw(d);

	//Draw the player's ship.
	player.draw(d);

	//Draw the bullets.
	for(i=0;i<bullets.size();++i)
		bullets[i]->draw(d);

	//Draw the weapons.
	for(i=0;i<weapons.size();++i)
		weapons[i]->draw(d);

	//Draw pink spores.
	for(i=0;i<pinkSpores.size();++i)
		d->drawImage(pinkSpore, pinkSpores[i].x, pinkSpores[i].y);

	//Draw fungi.
	for(i=0;i<fungi.size();++i)
		fungi[i]->draw(d);

	//Draw the boss fungus.
	if(bossTime)
		boss.draw(d);

	//Draw spores.
	for(i=0;i<spores.size();++i)
		spores[i]->draw(d);

	//Draw killer spores.
	for(i=0;i<killerSpores.size();++i)
		killerSpores[i]->draw(d);

	//Draw boulders.
	for(i=0;i<boulders.size();++i)
		d->drawImageRot(boulder, boulders[i].x, boulders[i].y, boulders[i].rot, 64.0f, 61.0f);

	//Draw foreground rocks.
	for(i=0;i<2;++i)
	{
		for(j=0;j<foreRocks[i].size();++j)
		{
			Rock& rock = foreRocks[i][j];

			d->drawImage(foreground[rock.image], rock.x, rock.y);
		}
	}

	//Draw the scanning value.
	for(i=0;i<fungiScanning.size();++i)
	{
		x = fungiScanning[i].x + ((Random::getGlobal().getRandomFloat() * 4.0f)-2.0f);
		y = fungiScanning[i].y + ((Random::getGlobal().getRandomFloat() * 4.0f)-2.0f);

		d->drawText(fungiScanning[i].name, font, x, y, 384.0f, 1.0f, 0.0f, 0.0f, 0.0f);
		d->drawSubImage(scanningBar, x, (y+80), 0.0f, 0.0f, fungiScanning[i].value, 1.0f);
	}

	///	Draw the number of fungi already scanned.
	{
		stringstream tempstr;

		tempstr << "Fungi Scanned: " << GameCounter::getInstance().getCount("FungiScanned") << "/" << fungusFactory.getNumFungusTypes();
		d->drawText(tempstr.str(), font, 32.0f, 7.0f, 512.0f, 1.0f, 0.0f, 0.0f, 0.0f);
		d->drawText(tempstr.str(), font, 30.0f, 5.0f, 512.0f, 1.0f, 1.0f, 1.0f, 1.0f);
	}

	if(shake > 0.0f)
	{
		d->translate(-shakeVal, 0.0f);
	}

	//If the player's won, wipe to white.
	if(winWipe < 1.0f)
	{
		if(winWipe > 0.0f)
			winWipe -= 0.01f;
		if(winWipe < 0.0f)
			winWipe = 0.0f;
		d->fillRect((winWipe * 1280.0f), 0, 1280.0f, 800.0f, 255, 255, 255, (1.0f-winWipe)*255);
	}
	else if(GameCounter::getInstance().getCount("FungiScanned") == fungusFactory.getNumFungusTypes())
	{
		winWipe -= 0.01f;
		Sounds::getInstance().playSound(Sounds::WinGame);
		helpText.trigger("Message from Command:\n Mission complete. Great job! Now come on home.", 480.0f);
	}

	//If the player's killed the boss, wipe to black.
	if(bossWipe < 1.0f)
	{
		if(bossWipe > 0.0f)
			bossWipe -= 0.01f;
		if(bossWipe < 0.0f)
			bossWipe = 0.0f;
		d->fillRect((bossWipe * 1280.0f), 0, 1280.0f, 800.0f, 0, 0, 0, (1.0f-bossWipe)*255);
	}

	//Draw any help text.
	helpText.draw(d);

	if(player.getDead())
		d->drawText("Mission Fail", bigFont, 480, 300, 512, 1.0f, 0.0f, 0.0f, 0.0f);

	if(paused && (winWipe >= 1.0f) && (bossWipe >= 1.0f))
		d->drawText("Paused", bigFont, 525, 390, 512, 1.0f, 0.0f, 0.0f, 0.0f);
}

//------------------------------------------------------------------------------
void GamePlay::keyDown(const string& action, const SDL_keysym& keyPressed)
{
	if(action == "Escape")
	{
		if(paused || player.getDead() ||(winWipe <= 0.0f) || (bossWipe <= 0.0f))
			nextState("TitleScreen");
		else
			paused = true;
	}
	else if(action == "Action")
	{
		if((winWipe <= 0.0f) || (bossWipe <= 0.0f))
			nextState("TitleScreen");
		else if(player.getDead())
			nextState("TitleScreen");
		else if(paused)
			paused = false;
	}
	else if(action == "P1Up")
		player.moveUp(true);
	else if(action == "P1Left")
		player.moveLeft(true);
	else if(action == "P1Right")
		player.moveRight(true);
	else if(action == "P1Down")
		player.moveDown(true);
}

//------------------------------------------------------------------------------
void GamePlay::keyUp(const string& action, const SDL_keysym& keyPressed)
{
	if(action == "P1Up")
		player.moveUp(false);
	else if(action == "P1Left")
		player.moveLeft(false);
	else if(action == "P1Right")
		player.moveRight(false);
	else if(action == "P1Down")
		player.moveDown(false);
}

//------------------------------------------------------------------------------
void GamePlay::addBullet(float x, float y, float direction)
{
	float xDir, yDir;

	xDir = -sinf((direction/360.0f) * 6.283185f);
	yDir = cosf((direction/360.0f) * 6.283185f);

	bullets.push_back(new Bullet(x+18.0f, y+20.0f, xDir, yDir, direction));

	Sounds::getInstance().playSound(Sounds::BulletFire);
}

//------------------------------------------------------------------------------
void GamePlay::addPinkSpore(float x, float y)
{
	float tempf;
	float x2, y2;
	PinkSpore spore;

	spore.x = x;
	spore.y = y;

	x2 = (player.getX()+(player.getWidth()*0.5f)) - x;
	y2 = (player.getY()+(player.getHeight()*0.5f)) - y;
	tempf = 1.0f/sqrtf((x2*x2) + (y2*y2));
	spore.xDir = x2 * tempf;
	spore.yDir = y2 * tempf;

	pinkSpores.push_back(spore);
}

//------------------------------------------------------------------------------
void GamePlay::displayEyeText()
{
	helpText.trigger("Message from Command:\n Is... that an eye?");
}

//------------------------------------------------------------------------------
void GamePlay::playerFungiCollision()
{
	int i, j;
	bool hit = false;
	float fX, fY, fX2, fY2;
	float bX, bY, bX2, bY2;
	const float pX = player.getX();
	const float pY = player.getY();
	const float pX2 = pX + player.getWidth();
	const float pY2 = pY + player.getHeight();

	for(i=0;i<fungi.size();++i)
	{
		fX = fungi[i]->getX();
		fY = fungi[i]->getY();
		fX2 = fX + fungi[i]->getWidth();
		fY2 = fY + fungi[i]->getHeight();

		if((pX2 > fX) && (pX < fX2) && (pY2 > fY) && (pY < fY2))
		{
			const vector<BoundingBox>& boxes = fungi[i]->getBounds();

			for(j=0;j<boxes.size();++j)
			{
				bX = fX + boxes[j].x;
				bY = fY + boxes[j].y;
				bX2 = bX + boxes[j].w;
				bY2 = bY + boxes[j].h;

				if((pX2 > bX) && (pX < bX2) && (pY2 > bY) && (pY < bY2))
				{
					hit = true;
					player.kill();
					shake = 1.0f;
					break;
				}
			}
			if(hit)
				break;
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::playerSporeCollision()
{
	int i;
	float sX, sY, sR;
	const float pX = player.getX();
	const float pY = player.getY();
	const float pX2 = pX + player.getWidth();
	const float pY2 = pY + player.getHeight();

	for(i=(spores.size()-1);i>=0;--i)
	{
		sX = spores[i]->getX();
		sY = spores[i]->getY();
		sR = spores[i]->getRadius();

		if((pX2 > (sX-sR)) &&
		   (pX < (sX+sR)) &&
		   (pY2 > (sY-sR)) &&
		   (pY < (sY+sR)))
		{
			delete spores[i];
			spores.erase(spores.begin()+i);

			weapons.push_back(new Weapon());
			Sounds::getInstance().playSound(Sounds::SporeCollide);
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::playerKillerSporeCollision()
{
	int i;
	float sX, sY, sR;
	const float pX = player.getX();
	const float pY = player.getY();
	const float pX2 = pX + player.getWidth();
	const float pY2 = pY + player.getHeight();

	for(i=(killerSpores.size()-1);i>=0;--i)
	{
		sX = killerSpores[i]->getX();
		sY = killerSpores[i]->getY();
		sR = killerSpores[i]->getRadius();

		if((pX2 > (sX-sR)) &&
		   (pX < (sX+sR)) &&
		   (pY2 > (sY-sR)) &&
		   (pY < (sY+sR)))
		{
			delete killerSpores[i];
			killerSpores.erase(killerSpores.begin()+i);

			player.kill();
			shake = 1.0f;

			break;
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::playerPinkSporeCollision()
{
	int i;
	float sX, sY, sR;
	const float pX = player.getX();
	const float pY = player.getY();
	const float pX2 = pX + player.getWidth();
	const float pY2 = pY + player.getHeight();

	for(i=(pinkSpores.size()-1);i>=0;--i)
	{
		sX = pinkSpores[i].x;
		sY = pinkSpores[i].y;
		sR = 9.0f;

		if((pX2 > (sX-sR)) &&
		   (pX < (sX+sR)) &&
		   (pY2 > (sY-sR)) &&
		   (pY < (sY+sR)))
		{
			pinkSpores.erase(pinkSpores.begin()+i);

			player.kill();
			shake = 1.0f;

			break;
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::playerBoulderCollision()
{
	int i;
	float sX, sY, sR;
	const float pX = player.getX();
	const float pY = player.getY();
	const float pX2 = pX + player.getWidth();
	const float pY2 = pY + player.getHeight();

	for(i=(boulders.size()-1);i>=0;--i)
	{
		sR = 46.0f;
		sX = boulders[i].x+sR;
		sY = boulders[i].y+sR;

		if((pX2 > (sX-sR)) &&
		   (pX < (sX+sR)) &&
		   (pY2 > (sY-sR)) &&
		   (pY < (sY+sR)))
		{
			boulders.erase(boulders.begin()+i);

			player.kill();
			shake = 1.0f;

			break;
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::bulletFungiCollision()
{
	int i, j, k;
	bool hit = false;
	float bX, bY, bX2, bY2, bR;
	float fX, fY, fX2, fY2;
	float cX, cY, cX2, cY2;

	for(k=(bullets.size()-1);k>=0;--k)
	{
		hit = false;
		bR = bullets[k]->getRadius();
		bX = bullets[k]->getX() - bR;
		bY = bullets[k]->getY() - bR;
		bX2 = bX + (2.0f*bR);
		bY2 = bY + (2.0f*bR);

		for(i=0;i<fungi.size();++i)
		{
			fX = fungi[i]->getX();
			fY = fungi[i]->getY();
			fX2 = fX + fungi[i]->getWidth();
			fY2 = fY + fungi[i]->getHeight();

			if((bX2 > fX) && (bX < fX2) && (bY2 > fY) && (bY < fY2))
			{
				const vector<BoundingBox>& boxes = fungi[i]->getBounds();

				for(j=0;j<boxes.size();++j)
				{
					cX = fX + boxes[j].x + 8;
					cY = fY + boxes[j].y + 8;
					cX2 = cX + boxes[j].w - 16;
					cY2 = cY + boxes[j].h - 16;

					if((bX2 > cX) && (bX < cX2) && (bY2 > cY) && (bY < cY2))
					{
						hit = true;

						fungi[i]->decrementHealth();
						if(fungi[i]->getHealth() <= 0)
						{
							//Generate explosion.
							explosions[explosionIndex].setStartSize(fungi[i]->getWidth(), fungi[i]->getHeight());
							explosions[explosionIndex].setPosition(fungi[i]->getX(), fungi[i]->getY());
							explosions[explosionIndex].start(15);
							++explosionIndex;
							if(explosionIndex >= MaxExplosions)
								explosionIndex = 0;

							delete fungi[i];
							fungi.erase(fungi.begin()+i);
							GameCounter::getInstance().incrementCount("FungiKilled");

							--fungiHealthCount;
							if(fungiHealthCount <= 0)
							{
								++Fungus::maxHealth;
								fungiHealthCount = 4;
							}

							//if(!Fungus::attacking)
							{
								++pinkSporeCount;
								if(pinkSporeCount == 1)
									helpText.trigger("Message from Command:\n Please be careful. This is a valuable ecosystem.");
								else if(pinkSporeCount == 8)
								{
									boulderCount = 60;
									boulderCount += (int)(Random::getGlobal().getRandomFloat() * 29.9999f);
									helpText.trigger("Message from Command:\n We're seeing some very strong seismic reactions here. Are you okay?");
								}
								else if(pinkSporeCount == 12)
								{
									Fungus::attacking = true;
									helpText.trigger("Message from Command:\n We're getting some strange readings up here. The fungi seem agitated?");
								}
								else if(pinkSporeCount == 20)
								{
									bossTime = true;
									Fungus::attacking = false;
									player.setBossTime();
									helpText.trigger("Message from Command:\n That can't be right! There's... something...");
								}
							}
						}
						delete bullets[k];
						bullets.erase(bullets.begin()+k);

						Sounds::getInstance().playSound(Sounds::FungusDie);

						shake = 1.0f;

						break;
					}
				}
				if(hit)
					break;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::bulletBossCollision()
{
	int i;
	float tempf;
	float bX, bY, bR;
	float fX, fY, fR;

	fX = boss.getEyeX();
	fY = boss.getEyeY();
	fR = boss.getEyeRadius();

	for(i=(bullets.size()-1);i>=0;--i)
	{
		bX = bullets[i]->getX() - fX;
		bY = bullets[i]->getY() - fY;
		bR = bullets[i]->getRadius();

		tempf = sqrtf((bX*bX) + (bY*bY));
		if(tempf < (bR+fR))
		{
			bullets.erase(bullets.begin()+i);

			Sounds::getInstance().playSound(Sounds::FungusDie);

			boss.decrementHealth();
			if(boss.getHealth() <= 0)
			{
				explosions[explosionIndex].setPosition(boss.getX(), boss.getY());
				explosions[explosionIndex].setStartSize(boss.getWidth(), boss.getHeight());
				explosions[explosionIndex].setDensity(32.0f);
				explosions[explosionIndex].start(15);
				++explosionIndex;
				if(explosionIndex > MaxExplosions)
					explosionIndex = 0;

				helpText.setColour(1.0f, 1.0f, 1.0f);
				helpText.trigger("Message from Command:\n This place is too dangerous. We're pulling you out.", 480.0f);

				bossTime = false;
				bossWipe -= 0.01f;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::playerFungiScanning()
{
	int i, j;
	bool hit = false;
	float fX, fY, fX2, fY2;
	float bX, bY, bX2, bY2;
	const float pX = player.getX() - 8.0f;
	const float pY = player.getY() - 8.0f;
	const float pX2 = pX + player.getWidth() + 16.0f;
	const float pY2 = pY + player.getHeight() + 16.0f;

	fungiScanning.clear();

	for(i=0;i<fungi.size();++i)
	{
		fX = fungi[i]->getX() - 64.0f;
		fY = fungi[i]->getY() - 64.0f;
		fX2 = fX + fungi[i]->getWidth() + 128.0f;
		fY2 = fY + fungi[i]->getHeight() + 128.0f;

		if(fungusFactory.getScannedVal(fungi[i]->getName()) >= 1.0f)
			continue;

		if((pX2 > fX) && (pX < fX2) && (pY2 > fY) && (pY < fY2))
		{
			const vector<BoundingBox>& boxes = fungi[i]->getBounds();

			for(j=0;j<boxes.size();++j)
			{
				bX = fX + boxes[j].x;
				bY = fY + boxes[j].y;
				bX2 = bX + boxes[j].w + 128.0f;
				bY2 = bY + boxes[j].h + 128.0f;

				if((pX2 > bX) && (pX < bX2) && (pY2 > bY) && (pY < bY2))
				{
					FungusScanData data;

					data.name = fungi[i]->getName();
					data.x = fX + 32.0f;
					data.y = fY + 32.0f;
					fungusFactory.incScannedVal(data.name);
					data.value = fungusFactory.getScannedVal(data.name);

					fungiScanning.push_back(data);
					if(fungusFactory.getScannedVal(data.name) < 1.0f)
						Sounds::getInstance().playSound(Sounds::Scanning);

					break;
				}
			}
		}
	}
}
