//	GamePlay.h - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAY_H_
#define GAMEPLAY_H_

#include "GameState.h"
#include "Player.h"
#include "FungusFactory.h"
#include "FadingText.h"
#include "ParticleGenerator.h"
#include "BossFungus.h"

#include <vector>
#include <stdint.h>

class Spore;
class Weapon;
class Bullet;
class KillerSpore;

///	The main gameplay state.
class GamePlay : public GameState
{
  public:
	///	Constructor.
	GamePlay(Application *app, bool easy);
	///	Destructor.
	~GamePlay();

	///	Draws the current state.
	void draw(Drawer *d);

	///	Called when a key down event is received.
	void keyDown(const std::string& action, const SDL_keysym& keyPressed);
	///	Called when a key up event is received.
	void keyUp(const std::string& action, const SDL_keysym& keyPressed);

	///	Called from Weapon when it fires a bullet.
	void addBullet(float x, float y, float direction);
	///	Called from Fungus when it fires a spore.
	void addPinkSpore(float x, float y);
	///	Called from BossFungus.
	void displayEyeText();

	///	Returns the name of this state.
	const std::string getName() const {return "GamePlay";};
  private:
	///	Helper method to handle player-fungi collisions.
	void playerFungiCollision();
	///	Helper method to handle player-spore collisions.
	void playerSporeCollision();
	///	Helper method to handle player-spore collisions.
	void playerKillerSporeCollision();
	///	Helper method to handle player-spore collisions.
	void playerPinkSporeCollision();
	///	Helper method to handle player-boulder collisions.
	void playerBoulderCollision();
	///	Helper method to handle bullet-fungi collisions.
	void bulletFungiCollision();
	///	Helper method to handle bullet-fungi collisions.
	void bulletBossCollision();
	///	Helper method to handle player-fungi collisions.
	void playerFungiScanning();
     
	///	The background rocks.
	static uint32_t background[3];
	///	The midground rocks.
	static uint32_t midground[3];
	///	The foreground rocks.
	static uint32_t foreground[3];
	///	The scanning bar image.
	static uint32_t scanningBar;
	///	The pink spore image.
	static uint32_t pinkSpore;
	///	The boulder image.
	static uint32_t boulder;
	///	The fungi explosion images.
	static uint32_t particles[3];

	///	The font to use.
	static uint32_t font;
	///	The font to use.
	static uint32_t bigFont;

	///	Struct representing a rock.
	struct Rock
	{
		///	The rock's position.
		float x;
		///	The rock's position.
		float y;
		///	The rock's image.
		uint32_t image;
	};

	///	Struct representing a pink spore.
	struct PinkSpore
	{
		///	The spore's position.
		float x;
		///	The spore's position.
		float y;
		///	The spore's direction.
		float xDir;
		///	The spore's direction.
		float yDir;
	};

	///	Struct representing a boulder.
	struct Boulder
	{
		///	The boulder's position.
		float x;
		///	The boulder's position.
		float y;
		///	The boulder's direction.
		float rot;
	};

	///	The far background rocks.
	std::vector<Rock> farRocks;
	///	The background rocks.
	std::vector<Rock> backRocks[2];
	///	The midground rocks.
	std::vector<Rock> midRocks[2];
	///	The foreground rocks.
	std::vector<Rock> foreRocks[2];

	///	Used to generate all our Fungi.
	FungusFactory fungusFactory;
	///	All the currently visible fungi.
	std::vector<Fungus *> fungi;
	///	Used to determine when to generate new fungi.
	int fungiCount;
	///	Used to increase the fungi's health the more of them the player kills.
	int fungiHealthCount;
	///	Struct holding the data about the fungi in range.
	struct FungusScanData
	{
		///	The name of the fungus type.
		std::string name;
		///	The position of the display.
		float x;
		///	The position of the display.
		float y;
		///	The amount it's been scanned by.
		float value;
	};
	///	Used to display how much of the fungi in range have been scanned.
	std::vector<FungusScanData> fungiScanning;
	///	How many explosions we can have.
	enum
	{
		MaxExplosions = 8
	};
	///	Explosions for when fungi die.
	ParticleGenerator explosions[8];
	///	Which explosion is next in line to explode.
	int explosionIndex;

	///	The boss fungus.
	BossFungus boss;
	///	Whether or not we've reached the boss.
	bool bossTime;

	///	All the active Spores.
	std::vector<Spore *> spores;
	///	All the active KillerSpores.
	std::vector<KillerSpore *> killerSpores;
	///	All the active PinkSpores.
	std::vector<PinkSpore> pinkSpores;
	///	Used to determine when to generate new spores.
	int sporeCount;
	///	Used to determine when Fungi start shooting at the player.
	int pinkSporeCount;

	///	All the active boulders.
	std::vector<Boulder> boulders;
	///	Used to determine when to generate new boulders.
	int boulderCount;

	///	All the active Weapons.
	std::vector<Weapon *> weapons;
	///	All the active Bullets.
	std::vector<Bullet *> bullets;

	///	The player.
	Player player;

	///	True if we've already generated a KillerSpore.
	bool killerGenerated;
	///	Helpful text.
	FadingText helpText;

	///	Shake the screen when something dies.
	float shake;

	///	The white wipe when the player scans all the fungi.
	float winWipe;
	///	The black wipe when the player kills the boss.
	float bossWipe;

	///	If we're paused or not.
	bool paused;

	///	Whether we're in easy mode or not.
	bool easyMode;
};

#endif
