//	ParticleGenerator.cpp - A particle generator.
//  ----------------------------------------------------------------------------
//	This file is part of 'Caverns of Light', a game.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ParticleGenerator.h"
#include "NiallsDrawingLib/Drawer.h"
#include "Random.h"

#include <cmath>
#include <cassert>

//------------------------------------------------------------------------------
ParticleGenerator::ParticleGenerator():
dirX(0.0f),
dirY(0.0f),
speed(1.0f),
startX(0.0f),
startY(0.0f),
startW(0.0f),
startH(0.0f),
density(1.0f),
lifespan(30.0f),
width(1.0f),
height(1.0f),
randomSize(false),
red(1.0f),
green(1.0f),
blue(1.0f),
alpha(1.0f),
timer(-1),
active(false),
newParticle(0.0f)
{
	
}

//------------------------------------------------------------------------------
ParticleGenerator::~ParticleGenerator()
{
	
}

//------------------------------------------------------------------------------
void ParticleGenerator::start(int time)
{
	timer = time;
	active = true;
}

//------------------------------------------------------------------------------
void ParticleGenerator::stop()
{
	active = false;
}

//------------------------------------------------------------------------------
void ParticleGenerator::move()
{
	int i;
	float dist;
	float tempf;

	if(active && (timer > -1))
	{
		--timer;
		if(timer == 0)
			active = false;
	}

	for(i=0;i<particles.size();++i)
		particles[i].move();

	//Get rid of any particles that should be dead.
	for(i=(particles.size()-1);i>=0;--i)
	{
		if(particles[i].lifespan < 0.0f)
			particles.erase(particles.begin()+i);
	}

	//Add any new particles.
	if(active)
	{
		newParticle -= density;
		while(newParticle <= 0.0f)
		{
			Particle p;

			i = (int)(Random::getGlobal().getRandomFloat() * (float(images.size())-0.5f));
			p.image = images[i];
			p.lifespan = lifespan;
			p.speed = speed + ((Random::getGlobal().getRandomFloat() - 0.5f) * 0.01f);
			if(randomSize)
				p.size = Random::getGlobal().getRandomFloat();
			else
				p.size = 1.0f;
			tempf = Random::getGlobal().getRandomFloat() * startW;
			p.x = startX - (width*p.size*0.5f) + tempf;
			tempf = Random::getGlobal().getRandomFloat() * startH;
			p.y = startY - (height*p.size*0.5f) + tempf;
			if((dirX == 0.0f) && (dirY == 0.0f))
			{
				p.dirX = (Random::getGlobal().getRandomFloat() - 0.5f) * 2.0f;
				p.dirY = (Random::getGlobal().getRandomFloat() - 0.5f) * 2.0f;
				dist = sqrtf((p.dirX*p.dirX) + (p.dirY*p.dirY));
				if(dist > 1.0f)
				{
					p.dirX /= dist;
					p.dirY /= dist;
				}
			}
			else
			{
				p.dirX = dirX + ((Random::getGlobal().getRandomFloat() - 0.5f) * 0.5f);
				p.dirY = dirY + ((Random::getGlobal().getRandomFloat() - 0.5f) * 0.5f);
				dist = sqrtf((p.dirX*p.dirX) + (p.dirY*p.dirY));
				if(dist > 1.0f)
				{
					p.dirX /= dist;
					p.dirY /= dist;
				}
			}
			particles.push_back(p);

			newParticle += 1.0f;
		}
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::draw(Drawer *d)
{
	int i;

	assert(images.size() > 0);

	for(i=0;i<particles.size();++i)
	{
		d->drawImage(particles[i].image,
					 particles[i].x,
					 particles[i].y,
					 (particles[i].lifespan/lifespan) * width * particles[i].size,
					 (particles[i].lifespan/lifespan) * height * particles[i].size,
					 red,
					 green,
					 blue,
					 (particles[i].lifespan/lifespan)*alpha);
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::draw(Drawer *d,
							 float viewX,
							 float viewY,
							 float viewW,
							 float viewH)
{
	int i;
	float x, y;
	float pX, pY;

	for(i=0;i<particles.size();++i)
	{
		pX = particles[i].x;
		pY = particles[i].y;
		x = (particles[i].x - viewX) * 32.0f;
		y = (particles[i].y - viewY) * 32.0f;

		if(((pX+1.0f) > viewX) && (pX < (viewX+viewW)) &&
		   ((pY+1.0f) > viewY) && (pY < (viewY+viewH)))
		{
			d->drawImage(particles[i].image,
						 (int16_t)(x-16.0f),
						 (int16_t)(y-16.0f),
						 (particles[i].lifespan/lifespan) * width * particles[i].size,
						 (particles[i].lifespan/lifespan) * height * particles[i].size,
						 red,
						 green,
						 blue,
						 (particles[i].lifespan/lifespan)*alpha);
		}
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::addImage(uint32_t im)
{
	images.push_back(im);
}

//------------------------------------------------------------------------------
void ParticleGenerator::setDirection(float x, float y)
{
	dirX = x;
	dirY = y;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setSpeed(float val)
{
	speed = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setPosition(float x, float y)
{
	startX = x;
	startY = y;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setStartSize(float w, float h)
{
	startW = w;
	startH = h;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setDensity(float val)
{
	density = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setLifespan(float val)
{
	lifespan = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setSize(float w, float h)
{
	width = w;
	height = h;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setRandomSize(bool val)
{
	randomSize = val;
}

//------------------------------------------------------------------------------
void ParticleGenerator::setColour(float r, float g, float b, float a)
{
	red = r;
	green = g;
	blue = b;
	alpha = a;
}
