//	ParticleGenerator.h - A particle generator.
//  ----------------------------------------------------------------------------
//	This file is part of 'Caverns of Light', a game.
//	Copyright (C) 2009  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PARTICLEGENERATOR_H_
#define PARTICLEGENERATOR_H_

#include <vector>
#include <stdint.h>

class Drawer;

///	A particle generator.
class ParticleGenerator
{
  public:
	///	Constructor.
	ParticleGenerator();
	///	Destructor.
	~ParticleGenerator();

	///Starts the particles being generated.
	/*!
		\time Milliseconds until the particles are stopped.
	 */
	void start(int time = -1);
	///	Stops the particles being generated.
	void stop();
	///	Returns whether or not the generator is active.
	bool isActive() const {return active;};

	///	Moves the particles.
	void move();

	///	Draws the particles.
	void draw(Drawer *d);
	///	Draws the particles (only draws particles w/in the view rectangle).
	void draw(Drawer *d, float viewX, float viewY, float viewW, float viewH);

	///	Adds a particle image.
	void addImage(uint32_t im);
	///	Sets the direction to move the particles.
	void setDirection(float x, float y);
	///	Sets the speed of the particles.
	void setSpeed(float val);
	///	Sets the position of the particles' start point.
	void setPosition(float x, float y);
	///	Sets the width and height of the particles' start point.
	void setStartSize(float w, float h);
	///	Sets the particles' density.
	void setDensity(float val);
	///	Sets the particles' lifespan.
	void setLifespan(float val);
	///	Sets the particles' size.
	void setSize(float w, float h);
	///	Randomises the particles' size.
	void setRandomSize(bool val);
	///	Sets the colour to tint the particles.
	void setColour(float r, float g, float b, float a);
  private:
	///	The particle image(s).
	std::vector<uint32_t> images;
	///	The particles' direction.
	float dirX;
	///	The particles' direction.
	float dirY;
	///	The particles' speed.
	float speed;
	///	The particles' start position.
	float startX;
	///	The particles' start position.
	float startY;
	///	The size of the particles' starting position.
	float startW;
	///	The size of the particles' starting position.
	float startH;
	///	The particles' density.
	float density;
	///	The particles' lifespan.
	float lifespan;
	///	The particles' size.
	float width;
	///	The particles' size.
	float height;
	///	Randomises the particles' size.
	bool randomSize;
	///	Particles' colour.
	float red;
	///	Particles' colour.
	float green;
	///	Particles' colour.
	float blue;
	///	Particles' colour.
	float alpha;
	///	Used to switch the particles off automatically.
	int timer;

	///	Whether or not the particles should be generated.
	bool active;

	///	Represents a single particle.
	struct Particle
	{
		///	The particle's image.
		uint32_t image;
		///	The particle's position.
		float x;
		///	The particle's position.
		float y;
		///	The particle's direction.
		float dirX;
		///	The particle's direction.
		float dirY;
		///	The particle's speed.
		float speed;
		///	The particle's lifespan.
		float lifespan;
		///	Random size factor.
		float size;

		///	Updates the particle's position.
		void move()
		{
			x += dirX * speed;
			y += dirY * speed;

			lifespan -= 0.1f;
		}
	};
	///	The active particles.
	std::vector<Particle> particles;

	///	Used to determine when to add more particles.
	float newParticle;
};

#endif
